import qualified Prelude
import Feldspar hiding ((**))



--------------------------------------------------------------------------------
-- Core language
--------------------------------------------------------------------------------

-- Iterative definition of modulus division
modulus :: (Ord a, Num a) => Data a -> Data a -> Data a
modulus a b = while (>=b) (subtract b) a

-- Count the number of occurrences of `a` in the array `arr`. The array length
-- is given by `l`.
count :: (Eq a, Show a) => Data Length -> Data a -> Data [a] -> Data Int
count l a arr = for 0 (l-1) 0 $ \i s ->
    (arr!i)==a ? (s+1,s)

test_count = count l 2 arr
  where
    as  = [1,2,3,4,5,4,3,2,1,2,3,4,5,6,5,4,3,2,1] :: [Int]
    arr = value as
    l   = value (Prelude.length as)
  -- Try with `eval` or `printCore`

-- The sum of the squares of the numbers `[1..n]`.
sumSq_core :: Data Int -> Data Int
sumSq_core n = for 1 n 0 $ \i s -> s + i*i



--------------------------------------------------------------------------------
-- Vector library
--------------------------------------------------------------------------------

-- The sum of the squares of the numbers `[1..n]`.
sumSq :: Data Int -> Data Int
sumSq n = sum $ map square (1...n)
  where
    square x = x*x

findFirst :: (a -> Data Bool) -> Vector a -> a
findFirst pred = head . dropWhile (not.pred)

findLast :: (a -> Data Bool) -> Vector a -> a
findLast pred = findFirst pred . reverse

test_findFirst = findFirst (<10) (vector [123,234,345,56,3,56,7,234])
  -- Try with `eval` or `printCore`

test_findLast = findLast (<10) (vector [123,234,345,56,3,56,7,234])
  -- Try with `eval` or `printCore`

(**) :: Num a => DVector a -> DVector a -> Data a
a ** b = sum $ zipWith (*) a b

convolution :: Num a => DVector a -> DVector a -> DVector a
convolution a b = indexed (length b) $ \n ->
   take (n+1) a ** reverse (take (n+1) b)

